'<TiCoBasic Header, Headerversion 001.001>
' Foldings                       = 27,88
'<Header End>
'ADBASIC-SYSTEM-FILE--V1.00--0000--0000--INC--PRO
'
'##############################################################################
'#                        ArincTico.inc                                       #
'#              INCLUDE-file for Arinc                                        #
'##############################################################################
'#                                                                            #
'#                                                                            #
'#  19.02.2013  mf   creation                                                 #
'#  22.02.2013  ms   add new functions                                        #
'#  12.03.2013  mf   update                                                   #
'#  04.11.2016  mf   Calc_Processdelay
'#                                                                            #
'# Attention:                                                                 #
'#   - Do not copy the first source line (containing                          #
'#     "'ADBASIC-SYSTEM-FILE...") to user files !                             #
'#     Otherwise the Syntax color may be not correct.                         #
'#   - This includefile requires ADbasic 5 or higher!                         #
'#                                                                            #
'##############################################################################

#if adwin_system=ADwin_ProII
#else
#error("This include file can only be used with the ADwin ProII")  
#endif

'<REGION> SYSTEM

'***************************************************************
''' Converts a process frequency [Hz] into processor ticks (processdelay or cycle time).
''' frequency:  frequency[Hz]
'''*************************************************************
Function Calc_Processdelay(frequency) as long
  
#if prozessor=tico1
  calc_processdelay=50000000/(frequency)
#else
  calc_processdelay=100000000/(frequency)
#endif
  
endfunction
'---------------------------------------------------------------
' If a low-priority process accesses a Pro-module it must not be
' interrupted by any other process !!!
'---------------------------------------------------------------

Sub Sub_Begin_Critical_Section() ''<HIDE>
#If PRIORITAET = LOW
  Begin_Critical_Section
#EndIf
EndSub

Sub Sub_End_Critical_Section() ''<HIDE>
#If PRIORITAET = LOW
  End_Critical_Section
#EndIf
EndSub

'******************************************************************************
'''Switches the LED on or off.
'''value: 
'''+  0 off
'''+  1 on
'''****************************************************************************
SUB Set_LED(value)
  out (80804000H, value)
ENDSUB

'******************************************************************************
'''Checks the LED.
'''+Return:
'''+   0 off
'''+   1 on
'''****************************************************************************
FUNCTION Check_LED() AS Long
  Check_LED=in(80804000h)
ENDFUNCTION

'*******************************************************************************
'''Creates an event for the master cpu.
'''*****************************************************************************
SUB Trigger_Event()
  out (108041E0h, 1)
ENDSUB

'<ENDREGION>

'<REGION> ARINC 429

'******************************************************************************
'''Does a Master Reset on the ARINC module.
'''+On a Master Reset data transmission and reception are immediately terminated,
'''+all three FIFOs are cleared as are the FIFO flags and the Status Register.
'''+The Control Register is not affected by a Master Reset.
'''****************************************************************************
SUB ARINC_Reset()
  dim ARINC_busy as long
  OUT(4100h, 0)       'nMReset = 0
  nops(8)
  out(4100h,1)
  nops(8)
  'CR15=0 
  'Write Control Word Register                                                               
  out(0,00020h)
  nop
  nop
  nop
  do
    ARINC_busy=in(4105h)
  until (ARINC_busy<>1)   'angeforderte Daten gltig / Ready fr neuen Befehl 
endSub

'******************************************************************************
'''Configures the transmit settings on the specified ARINC module.
'''clk: data rate
'''+      0: 100kHz (high speed)
'''+      1: 12.5kHz (low speed)
'''parity_enable: 
'''+      0: No parity, bit 32 is part of transmitted data 
'''+      1: Bit 32 is parity bit
'''parity:  
'''+      0: Odd parity
'''+      1: Even parity
'''****************************************************************************
SUB ARINC_Config_Transmit(clk,parity_enable,parity)
  dim ARINC_busy as long
  dim old as long
  'Control Register (CR) lesen (16Bit)
  out(3,1)
  nop
  nop
  nop

  do 
    old=in(0)
  until (old>=0)
  
  old =old AND 0CFEFH
  out(0,(old or shift_left(clk,13)or shift_left(parity,12) or shift_left(parity_enable,4)))  'clk
  nop
  nop
  nop
  do
    ARINC_busy=in(4105h)
  until (ARINC_busy<>1)   'angeforderte Daten gltig / Ready fr neuen Befehl 
ENDSUB

'******************************************************************************
'''Configures the receiver settings on the specified ARINC module.
'''channel: number (1,2) of receive channel
'''clk: data rate 
'''+      0: 100kHz (high speed)
'''+      1: 12.5kHz (low speed)
'''enable_label: flag for label matching
'''+      0: off, all incoming values are read into FIFO
'''+      1: on, only values matching the label will be read into FIFO
'''enable_decoder:
'''+             0 = off, no extra decoding of ARINC bit 9 and 10 (SDI)
'''+             1 = on, only if matching the decoding bits 9 and 10, the value will be read into FIFO
'''decoder_value: 0..3, this must match the value from ARINC bits 9 and 10 that the value will be read into FIFO 
'''****************************************************************************
SUB ARINC_Config_Receive(channel,clk,enable_label, enable_decoder, decoder_value)
  dim ARINC_busy as long
  dim old as long
  'Control Register (CR) lesen (16Bit)
  out(3,1)
  nop
  nop
  nop
  
  do
    old=in(0)
  until(old>=0)
 
  if (channel=1) then 
    old =old AND 0FE3AH
    old =old or shift_left(clk,0) or shift_left(enable_Label,2) or shift_left(enable_decoder, 6) or shift_left(decoder_value, 7)
  else
    old =old AND 0B1F7H
    old =old or shift_left(clk,14)  or shift_left(enable_Label,3) or shift_left(enable_decoder, 9) or shift_left(decoder_value, 10) 
  endif
  
  'Write Control Word Register
  out(0,old)
  nop
  nop
  nop                                                                      
  
  do
    arinc_busy=in(4105h)
  until (ARINC_busy<>1)   'angeforderte Daten gltig / Ready fr neuen Befehl 
ENDSUB

'******************************************************************************
'''Gets the status of the transmit FIFO of the ARINC module.
'''+The transmitters FIFO can store 32 words maximum.
'''+return value: 
'''+      0: Transmit FIFO is not full.
'''+      1: Transmit FIFO is full.
'''****************************************************************************
Function ARINC_Transmit_Fifo_Full() as long
  ARINC_Transmit_Fifo_Full= (shift_right(in(4104h),2) AND 1) xor 1  
ENDSUB

'Check ob Daten gesendet
'******************************************************************************
'''Gets the status of the transmit FIFO of the ARINC module.
'''+The transmitters FIFO can store 32 words maximum. 
'''+return value: 
'''+      0: Transmit FIFO contains data.
'''+      1: Transmit FIFO is empty.
'''****************************************************************************
Function ARINC_Transmit_Fifo_Empty() as long
  ARINC_Transmit_Fifo_Empty=in(4104h) AND 1
ENDSUB

'******************************************************************************
'''Writes a 32 bit value into the transmitter FIFO of the ARINC module.
'''+The transmitter FIFO can store 32 words maximum and ignores attempts to load
'''+ additional data if full.
'''+Use P2_ARINC_Transmit_Fifo_Full to check for free space in the transmitter
'''+Fifo before writing a new value into.
''' value:  32 bit data word to be transmitted.
'''+ Bit 0...7 Label 1..8
'''+ Bit 9...10 SSM 1..2
'''+ Bit 11..12 SSM
'''+ Bit 13...31 Daten 1..19
'''****************************************************************************
SUB ARINC_Write_Transmit_Fifo(value) 
  dim ARINC_busy as long
  
  out(1,value)
  nop
  nop
  nop   
  
  
  do
    ARINC_busy=in(4105h)
  until (ARINC_busy<>1)   
  
EndSUB

'******************************************************************************
'''Enables or disables transmitting data from the transmit FIFO.
'''module: module address (1..15)
'''enable: transmitter flag
'''+      0: disable transmitting
'''+      1: enable transmitting
'''****************************************************************************
SUB ARINC_Transmit_Enable(enable)
  out(4101h,enable)
ENDSUB    

'******************************************************************************
'''Gets status of the receiver (IN) FIFO of the ARINC module.
'''+The receiver's FIFO can store 32 words maximum. 
'''+return value: 
'''+      0: Receiver FIFO has at least one value
'''+      1: Receiver FIFO is empty
'''channel: number (1,2) of receive channel
'''****************************************************************************
Function ARINC_Receive_Fifo_Empty(channel) as long
  ARINC_Receive_Fifo_Empty=(in(4101h+channel) AND 1) 'Problem wenn channel variabel ? peter fragen ob OK.
ENDSUB

'******************************************************************************
'''Gets a 32 bit value from the receiver FIFO of the ARINC module.
'''+The receivers FIFO can store 32 words maximum.
'''+Use P2_ARINC_Receive_Fifo_Empty to check for data in the receiver Fifo before
'''+reading a new value. 
'''+return value: 32 bit received data out of FIFO
'''+ Bit 0...7 Label 1..8
'''+ Bit 8 Parity
'''+ Bit 9...10 SSM 1..2
'''+ Bit 11..12 SSM
'''+ Bit 13...31 Daten 1..19
'''channel: number (1,2) of receive channel
'''****************************************************************************
FUNCTION ARINC_Read_Receive_Fifo(channel) as long
  dim ARINC_busy as long
  dim ReceiverNoBYTE1 as long
  dim BYTE1_lowvalues, BYTE2_highvalues as long
  
    
  out(4,shift_left(channel-1,1))
  nop
  nop
  nop
 
  'read low word
  do
    ARINC_busy=in(4105h)
  until (ARINC_busy<>1)   'angeforderte Daten gltig / Ready fr neuen Befehl 
  

  ARINC_Read_Receive_Fifo= in(1)

ENDSUB

'******************************************************************************
'''Sets 16 labels in the receiver label memory of the ARINC module.
'''+All 16 labels should be set.
'''channel: number (1,2) of receive channel
'''labels: Array with 16 labels for a receive channel. Each label has 8 bit length.
'''array_start_index: Index 1..n of the first value in the labels array to be read.
'''****************************************************************************
SUB ARINC_Set_Labels(channel , labels[],array_start_index)
  dim ARINC_busy as long
  dim i as long
  dim control_register_merken as long 'cr_tmp ist zum Wiederherstellen des Control Registers nachdem die Labels gelesen wurden
  'flipLabel Variablen
  dim bit1, bit2, bit3, bit4, bit5, bit6, bit7, bit8 as long
  dim ARINC_label as long

  'Set SEL = 1; nEN1 = receiver 1;  nEN2 = receiver 2 = EN1=1
  '(Bit 0) SEL = 1 erforderlich laut Doku S.6 READING LABELS
  'Control Register (CR) lesen (16Bit)
  out(3, 1) 
  nop
  nop
  nop
 
  'read low word
  do  
    control_register_merken= in(0)
  until (control_register_merken>=0) 

  'Set CR1 = 1 ;CR2/3 = 0
  'Write Control Word Register
  out(0h, 0010b)                                                                      
  nop
  nop
  nop
  do
    ARINC_busy=in(4105h)
  until (ARINC_busy<>1)   'angeforderte Daten gltig / Ready fr neuen Befehl 
  'Schreibe 16 Labels von Receiver 1/2 in die Label Memorys
  'Load 16 labels into the memory
  for i=0 to 15
    
    'flip label
    ARINC_Label = labels[i+array_start_index] AND 0FFh
     
    bit1 = shift_right(ARINC_label, 7) AND 1 
    bit2 = shift_right(ARINC_label, 5) AND 2
    bit3 = shift_right(ARINC_label, 3) AND 4
    bit4 = shift_right(ARINC_label, 1) AND 8
  
    bit5 = shift_left(ARINC_label, 1) AND 16 
    bit6 = shift_left(ARINC_label, 3) AND 32
    bit7 = shift_left(ARINC_label, 5) AND 64
    bit8 = shift_left(ARINC_label, 7) AND 128
  
    '
    ARINC_Label =  bit1 OR bit2 OR bit3 OR bit4 OR bit5 OR bit6 OR bit7 OR bit8
    
    
    
    out(05h+channel, ARINC_Label)
    nop
    nop
    nop
    do
      ARINC_busy=in(4105h)
    until (ARINC_busy<>1)   'angeforderte Daten gltig / Ready fr neuen Befehl 
  next i
 
  'Wiederherstellen des Control Registers und Sicherstellen das CR1=0 (normal operation) ist.
  'Write Control Word Register
  out(0h, control_register_merken)                                                                      
  nop
  nop
  nop
  do
    ARINC_busy=in(4105h)
  until (ARINC_busy<>1)   'angeforderte Daten gltig / Ready fr neuen Befehl 
EndSub

'******************************************************************************
'''Creates the ARINC 32 bit value out of the 4 parts SSM, DATA, SDI and Label.
'''+ After calling this function you can use P2_ARINC_Write_transmit_FIFO to write it into the transmitter FIFO. 
'''+ return value: 32 bit ARINC data
'''+ Bit 0...7 Label 1..8
'''+ Bit 8..9 SDI 1..2
'''+ Bit 10...29 Daten 1..19
'''+ Bit 29...30 SSM 1..2
'''+ Bit 31 Parity
'''arinc_label: (0..255) 8 bits ARINC label
'''ssm: (0..3) 2 bits Sign / Status Matrix
'''sdi: (0..3) 2 bits Source / Destination Identifier
'''data: (0..524287) 19 bits data (7FFFF Hex)
'''****************************************************************************
FUNCTION ARINC_Create_Value32(arinc_label, ssm, sdi, data) as long
  'ARINC_Create_value32 = ARINC_label OR shift_left(SDI, 8) OR shift_left(data, 10) OR shift_left(SSM, 29)
  ARINC_Create_value32 = ARINC_label OR shift_left(SDI, 11) OR shift_left(data, 13) OR shift_left(SSM, 9)

ENDFUNCTION

'******************************************************************************
'''Splits the ARINC 32 bit value into the 4 parts SSM, DATA, SDI and Label.
'''value: ARINC 32 bit value wich should be separatet into 4 variables
'''arinc_label: (0..255) 8 bits ARINC label
'''ssm: (0..3) 2 bits Sign / Status Matrix
'''sdi: (0..3) 2 bits Source / Destination Identifier
'''data: (0..524287) 19 bits data (7FFFF Hex)
'''parity: 0 = OK , 1 = Error
'''****************************************************************************
SUB ARINC_Split_Value32(value,  arinc_label, ssm, sdi, data,parity)
  arinc_label = value AND 0FFh
  SDI =shift_right(value, 11) AND 03h
  data = shift_right(value, 13) AND 07FFFFh
  SSM = shift_right(value, 9) AND 03h
  Parity=shift_right(value,8) AND 1
ENDSUB

'<ENDREGION>


