'<TiCoBasic Header, Headerversion 001.001>
' Foldings                       = 29,124,398,460,546
'<Header End>
'ADBASIC-SYSTEM-FILE--V1.00--0000--0000--INC--PRO
'
'##############################################################################
'#                        SPI_TiCo.inc                                        #
'##############################################################################
'#                                                                            #
'#                                                                            #
'#  05.11.2012  mf   creation                                                 #
'#  17.04.2014  ap   Digin_Filter_Init, DigProg_Set_IO_Level (Dio32TiCo.inc)  #
'#  29.04.2014  mf   Get_Digout_Long                                          #
'#  27.02.2015  mf   bugfix SPI_Master_Set_Clk_Wait                           #
'#  04.11.2016  mf   Calc_Processdelay                                        #
'#                                                                            #
'#                                                                            #
'# Attention:                                                                 #
'#   - Do not copy the first source line (containing                          #
'#     "'ADBASIC-SYSTEM-FILE...") to user files !                             #
'#     Otherwise the Syntax color may be not correct.                         #
'#   - This includefile requires ADbasic 5 or higher!                         #
'#                                                                            #
'##############################################################################


#if adwin_system=ADwin_ProII
#else
#error("This include file can only be used with the ADwin ProII")  
#endif

'<REGION> System

'***************************************************************
''' Converts a process frequency [Hz] into processor ticks (processdelay or cycle time).
''' frequency:  frequency[Hz]
'''*************************************************************
Function Calc_Processdelay(frequency) as long
  
#if prozessor=tico1
  calc_processdelay=50000000/(frequency)
#else
  calc_processdelay=100000000/(frequency)
#endif
  
endfunction
'---------------------------------------------------------------
' If a low-priority process accesses a Pro-module it must not be
' interrupted by any other process !!!
'---------------------------------------------------------------

Sub Sub_Begin_Critical_Section() ''<HIDE>
#If PRIORITAET = LOW
  Begin_Critical_Section
#EndIf
EndSub

Sub Sub_End_Critical_Section() ''<HIDE>
#If PRIORITAET = LOW
  End_Critical_Section
#EndIf
EndSub

'******************************************************************************
'''Switches the LED on or off.
'''value: 
'''+  0 off
'''+  1 on
'''****************************************************************************
SUB Set_LED(value)
  out (80804000H, value)
ENDSUB

'******************************************************************************
'''Checks the LED.
'''+Return:
'''+   0 off
'''+   1 on
'''****************************************************************************
FUNCTION Check_LED() AS Long
  Check_LED=in(80804000h)
ENDFUNCTION


'*******************************************************************************
'''Activates or deactivates the external Event input.
'''value:
'''+     0 deactivate
'''+     1 activate
'''*****************************************************************************
SUB Event_Enable(value)
  'DIM old AS Long
  '
  'old=in(80804800h)
  'old =(old AND 0FFFFFFFEH) or value
  'out (80804800h, old)
  
  out(80804800h,(in(80804800h) AND 0FFFFFFFEH) or value)

ENDSUB

'*******************************************************************************
'''Configures the external Event input.
'''min_hold:
'''+     0 = 15 ns (default) 
'''+     1 = 50 ns
'''edge:
'''+      1 rising edge (default)
'''+      2 falling edge
'''+      3 rising & falling edge
'''       prescale: 1 to 255
'''****************************************************************************
SUB Event_Config(min_hold,edge,prescale)
  out (80804800h,(in(80804800h) AND 0FFFFFFE3H) or shift_left((min_hold AND 1),4) or shift_left((edge AND 3),2))
  out (80804810h,prescale AND 0FFH)
ENDSUB

'*******************************************************************************
'''Creates an event for the master cpu.
'''*****************************************************************************
SUB Trigger_Event()
  out (108041E0h, 1)
ENDSUB

'<ENDREGION>

'<REGION> DIO

'******************************************************************************
'''Initialises the digital I/O module (channel 0 - 31).
'''value: each bit corresponds to 8 channel
'''+     0 = input
'''+     1 = output
'''****************************************************************************
SUB DigProg(value)
  out (4100H, value)
ENDSUB

'******************************************************************************
'''Sets the digital outputs (channel 0 - 31).
'''value: each bit corresponds to a channel
'''****************************************************************************
SUB Digout_Long(value)
  out (0, value)
ENDSUB

'******************************************************************************
'''Reads back of the output value of the digital
'''+outputs (channel 0 - 31).
'''****************************************************************************
FUNCTION Get_Digout_Long() AS Long
  nop
  nop
  Get_Digout_Long=in(0)
ENDFUNCTION

'******************************************************************************
'''Reads the digital inputs (channel 0 - 31).
'''****************************************************************************
FUNCTION Digin_Long() AS Long
  Digin_Long=in(100H)
ENDFUNCTION

'******************************************************************************
'''Sets and resets selected digital outputs.
'''set: Bitstring indicating the outputs which should be set
'''clear: Bitstring indicating the outputs which should be reset
'''****************************************************************************
SUB Digout_Bits(set,clear)
  out(20H,set)
  out(21H,clear)
ENDSUB

******************************************************************************
'''Sets selected digital outputs.
'''set: Bitstring indicating the outputs which should be set
'''***************************************************************************
SUB Digout_Set(set)
  out(20H,set)
ENDSUB

******************************************************************************
'''Resets selected digital outputs.
'''clear: Bitstring indicating the outputs which should be reset
'''***************************************************************************
SUB Digout_Reset(clear)
  out(21H,clear)
ENDSUB

'******************************************************************************
'''Sets or resets one channel of a digital output.
'''channel: Number of the output to be set (0..31)
'''value:
'''+     0 = reset channel
'''+     1 = set channel
'''****************************************************************************
SUB Digout(channel,value)
  out((0021h-(value)),(shift_left(1,(channel))))
ENDSUB

'******************************************************************************
'''Clears Digin Fifo.
'''****************************************************************************
SUB Digin_FIFO_Clear()
  out(80804880h, 0) 'FIFO clear
ENDSUB

'******************************************************************************
'''Clears Digout Fifo.
'''****************************************************************************
SUB Digout_FIFO_Clear()
  out(80804880h, 0) 'FIFO clear
ENDSUB

'******************************************************************************
'''Enables Digin Fifo.
'''pattern: each bit corresponds to a channel
'''****************************************************************************
SUB Digin_FIFO_Enable(pattern)
  out(80804890h, pattern) 'FIFO clear
ENDSUB

'******************************************************************************
'''Determines which input channels the edge detection unit will be monitored.
'''pattern: each bit corresponds to a channel
'''****************************************************************************
SUB Digout_FIFO_Enable(pattern)
  out(80804890h, pattern) 'FIFO clear
ENDSUB

'******************************************************************************
'''Returns the number of saved value pairs in the FIFO of the edge detection unit.
'''****************************************************************************
FUNCTION Digin_FIFO_Full() as Long
  Digin_FIFO_Full=in(80804880h) AND 0FFFFH
ENDFUNCTION

'******************************************************************************
'''Returns the number of free value pairs in the edge output FIFO.
'''****************************************************************************
FUNCTION Digout_FIFO_Empty() as Long
  Digout_FIFO_Empty=in(80804882h)
ENDFUNCTION

'******************************************************************************
'''Sets the FIFO operation mode: input with edge detection or edge output.
'''value:
'''+    0 Use FiFo as Input (default)
'''+    1 Use FiFo as Output absolute
'''+    2 reserved 
'''+    3 Use FiFo as Output relativ
'''+     Mode 1 available from Version 
'''+     DIO TiCo >=E0.3 or DIO TiCo-100 >=E0.1
'''****************************************************************************
SUB Dig_FIFO_Mode(value)
  out(80804870h, value) 
ENDSUB

'******************************************************************************
'''Starts output pattern.
'''****************************************************************************
SUB Digout_FIFO_Start()
  out(4860h, 1)
ENDSUB

'******************************************************************************
'''Reads one value pair from the FIFO of the edge detection unit and writes them into 2 variables.
'''value_by_ref: Variable where the level status bit pattern is written
'''+Each level status bit corresponds to a digital input (see table below).
'''timestamp_by_ref: Variable where the time stamp is written.
'''****************************************************************************
SUB Digin_FIFO_Read(value_by_ref,timestamp_by_ref)
  SUB_BEGIN_CRITICAL_SECTION
  value_by_ref=in(310H)
  timestamp_by_ref=in(310H)
  SUB_END_CRITICAL_SECTION
ENDSUB

'******************************************************************************
'''Writes value pairs into the output edge FIFO.
'''value: Variable where the level status bit pattern is written
'''timestamp: Variable where the time stamp is written
'''****************************************************************************
SUB Digout_FIFO_Write(value,timestamp)
  SUB_BEGIN_CRITICAL_SECTION
  out(310H,value)
  out(310H,timestamp)
  SUB_END_CRITICAL_SECTION
ENDSUB

'''******************************************************************************
'''Reads 100 MHz FIFO Timer.
'''+ This timer runs synchronous to the 50 MHz TiCo Timer.
'''+ In difference to the TiCo timer, this timer will not be set to 0 while core reset.
'''******************************************************************************
FUNCTION Digin_FIFO_Read_Timer() AS LONG
  Digin_FIFO_Read_Timer = IN(0311H)
ENDFUNCTION

'******************************************************************************
'''Reads 100 MHz FIFO Timer.
'''+ This timer runs synchronous to the 50 MHz TiCo Timer.
'''+ In difference to the TiCo timer, this timer will not be set to 0 while core reset.
'''*****************************************************************************
FUNCTION Digout_FIFO_Read_Timer() AS LONG
  Digout_FIFO_Read_Timer = IN(0311H)
ENDFUNCTION

'******************************************************************************
'''Reads one of the Latch Registers, which detect, if rising / falling edges
'''+ occured on the digital inputs (channel 0 - 31).
'''+ (These registers are cleared on reading)
'''edge:
'''+     0 latch register for the falling edges
'''+     1 latch register for the rising edges
'''****************************************************************************
FUNCTION Digin_Edge(edge) AS Long
  if (edge = 1) then
    Digin_Edge=in(300H) ' read lath register for rising edges
  else
    Digin_Edge=in(301H) '  ...  for falling edges
  endif
ENDFUNCTION

**********************************************************************
'''Initialises the differential digital channels 0..11 as input or output.
'''pattern: each bit corresponds to 1 channel
'''+     bit=0: set channel as input
'''+     bit=1: set channel as output
'''+     bit no. = channel no. (only channels 0..11)
'''****************************************************************************
SUB DigProg_Bits(pattern)             
  out(4101H,pattern)
ENDSUB

'******************************************************************************
'''Sets the voltage level of the digital channels in groups of 8 to a defined value.
'''+After power-up, the previously set voltage levels are active. For safety reasons you should set 
'''+the channel levels again in the Init: section with DigProg_Set_IO_Level before starting the output.
'''group:
'''+     0=channel (1..8)
'''+     1=channel (9..16)
'''+     2=channel (17..24)
'''+     3=channel (25..32)
'''level: 0..255 (see online help for details)
'''****************************************************************************
SUB DigProg_Set_IO_Level(group, level)
  dim bit,poti,i as long
  
  '#if prozessor=tico2
  poti=0
  sub_begin_critical_section
  out (10801500h,77359400h) 'Poti ID
  nops(4)                   
 
  for i=0 to 31
    if ((i=0) or (i=16)) then 'stack
      bit=in(10801502h) 'stackbit
      nops(20)
    endif
        
    bit=in(10801502h)
    nops(20)
    poti=poti or (shift_left(bit and 1,31-i))
  next
  
  poti= (poti AND not(shift_left(0FFH,(group)*8))) or shift_left(255-(level AND 0FFH),(group)*8)  
  for i=0 to 31
    if ((i=0) or (i=16)) then 'stack bit                   
      out (10801502h,0) ' 'Poti 2 Stack Bit
      nops(10)
    endif
    
    out (10801502h,shift_right(poti, 31-i) and 1) ' 'Poti out
    nops(10)
  next i
  
  out (10801500h,0) 'Poti ID disable            'Poti ID => Disable
  nops(8)
  sub_end_critical_section
  '#else
  
  '#endif
ENDSUB

''******************************************************************************
'''Sets spike filter for all digins.
'''filter_value: Filtering duration, given in cycles (1..65535) of 20 ns length.
'''+The value 0 disables the filter.
'''+The filtering duration should be a bit longer than the expected width of spikes.
''''****************************************************************************
SUB Digin_Filter_Init(filter_value)
  out(80804876h, Filter_value) 
  nop
  nop
  nop
ENDSUB

'<ENDREGION>

'<REGION> SPI common

'*******************************************************************************
'''Sets the SPI module operation mode.
'''mode: key number to set the operation mode
'''+  1: digital inputs/outputs only, refers to module Pro II-DIO-32-TiCo
'''+  2: SPI interface, both channels SPI slaves
'''+  3: SPI interface, channel 1 SPI master, channel 2 SPI slave
'''+  4: SPI interface, both channels SPI master
'''+  5: SPI interface, both channels SPI slaves, shared clock input SCLK1
'''*****************************************************************************
SUB SPI_Mode(mode)
  dim imode as long
  if (mode=1) then imode=1 
  if (mode=2) then imode=6
  if (mode=3) then imode=12
  if (mode=4) then imode=24
  if (mode=5) then imode=6+64
  if (mode=6) then imode=6+64+32 'ADZL 0507    
  out(80805A00H,imode)
ENDSUB

'******************************************************************************
'''Configures an SPI channel, either SPI master and SPI slave.
'''channel: SPI channel (1..2)  
''' mode: SPI master mode for CPOL (clock polarity) und CPHA (clock phase)
'''+    0: CPOL=0 CPHA=0 
'''+    1: CPOL=0 CPHA=1
'''+    2: CPOL=1 CPHA=0
'''+    3: CPOL=1 CPHA=1
'''bitlength: number (1..64) of transferred bits in an SPI message
'''data_order: sequence of bits in an SPI message
'''+    0: MSB first
'''+    1: LSB first
'''ss_select: Level to set a slave-select line active
'''+    0: active low
'''+    1: active high
'''******************************************************************************
SUB SPI_Config(channel,mode,bitlength,data_order,ss_select)
  dim imode as long
  
  if (mode=0) then imode=3 
  if (mode=1) then imode=0
  if (mode=2) then imode=11  
  if (mode=3) then imode=8
  
  imode=imode or shift_left(data_order AND 1,4)
  imode=imode or shift_left(ss_select AND 1,2)
  
  out(39F7H+channel,0) 'slave fifo disable
  nop
  nop
  out(383Fh + channel ,imode)
  out(381Fh + channel ,bitlength - 1)
  out(39F7H+channel,1) 'slave fifo enable
  nop
  nop
  nop
ENDSUB

'<ENDREGION>

'<REGION> SPI Slave

'******************************************************************************
'''Configures the settings of an SPI Slave; works supplemental to SPI_Config.
'''channel: SPI channel (1..2) 
'''mode: Set mode "store number of received bits"
'''+      0: store SPI message only in slave infifo
'''+      1: store both number of received bits and SPI message in slave infifo
'''****************************************************************************
SUB SPI_Slave_Config(channel,mode)
  out(80805A03h+channel,mode)
ENDSUB

'******************************************************************************
'''Writes a 32 bit value into output FIFO of an SPI slave.
'''channel: SPI channel (1..2) 
'''+Check first with SPI_Slave_OutFifo_Empty, if the Fifo has enough space.
'''+ The output Fifo can hold 18 values of 32 bit. If SPI bit length exceeds
'''+ 32 bit, store values in pairs: lower word first, then higher word.
'''value: 32 bit value
'''****************************************************************************
SUB SPI_Slave_OutFifo_Write(channel, value) 
  if (channel=1) then 
    out(3900H,value)
  else
    out(3901H,value)
  endif
  nop
ENDSUB

'******************************************************************************
'''Returns the number of free places in the output Fifo of an SPI slave.
'''channel: SPI channel (1..2) 
'''****************************************************************************
FUNCTION SPI_Slave_OutFifo_Empty(channel) as Long
  SPI_Slave_OUTFIFO_Empty=((Shift_Right(in(80805A07H+channel),16)) and 0000001Fh )
ENDFUNCTION

'******************************************************************************
'''Returns the number of used places (= received 32 bit values) in the input
'''+ Fifo of an SPI slave.
'''channel: SPI channel (1..2) 
'''****************************************************************************
FUNCTION SPI_Slave_InFifo_Full(channel) as Long  
  SPI_Slave_InFIFO_Full=in(80805A07H+channel) AND 0000001Fh
ENDFUNCTION

'******************************************************************************
'''Clears the input Fifo and/or output Fifo of an SPI slave.
'''channel: SPI channel (1..2) 
'''pattern: bit pattern to define the Fifos being cleared
'''+ Bit 0: Input Fifo (MOSI)
'''+      Bit=0: leave Fifo unchanged; bit=1: clear Fifo
'''+ Bit 1: Output Fifo (MISO)
'''+      Bit=0: leave Fifo unchanged; bit=1: clear Fifo
'''****************************************************************************
SUB SPI_Slave_Clear_Fifo(channel,pattern)  
  out(80805A7FH+channel,pattern)
  nop
  nop
  nop
  nop
ENDSUB

'******************************************************************************
'''Reads one 32 bit value from input Fifo of an SPI slave.
'''+Check first with SPI_Slave_InFifo_Full, if the Fifo contains any data.
'''+ The input Fifo can hold 18 values of 32 bit. Each SPI message consists of 1..3
'''+ 32 bit values, depending on SPI bit length and whether the number of received bits
'''+ is stored. The order of values is:
'''+ - number of received bits (if set with SPI_Config)
'''+ - lower word of SPI message
'''+ - upper word of SPI message (if SPI bit length exceeds 32 bit.
'''channel: SPI channel (1..2)
'''****************************************************************************
Function SPI_Slave_InFifo_Read(channel) as long
  if (channel=1) then 
    SPI_Slave_InFIFO_Read=in(3900H)
  else
    SPI_Slave_InFIFO_Read=in(3901H)
  endif

ENDFunction

'<ENDREGION>

'<REGION> SPI Master
 
'******************************************************************************
'''Starts data transfer of an SPI master. If configured, also activates
'''+the slave-select line of the SPI master.
'''+Data transfer runs in both directions: Master sends an SPI message and also
'''+receives the slave's answer.
'''channel: SPI channel (1..2) 
'''****************************************************************************
SUB SPI_Master_Start(channel)
  out(37FFH+channel,111B)
  nop
  nop
  nop
  nop
ENDSUB

'see also SPI_Config and SPI_Mode
'**************************************************************************
'''Configures the settings of an SPI master; works supplemental to SPI_Config.
'''channel: SPI channel (1..2) 
'''clk_factor: Factor (2..2500) to determine the clock frequency f_SPI:
'''+            f_SPI = 25 MHz / clk_factor
'''+            Example: clk_factor=250 -> f_SPI = 25 MHz / 250 = 100 kHz
''' miso_delay: delay time for reading an SPI message in units (0..255) of 20 ns.
'''+            Delay time range is 0..5.1 s
''' ss_manual: activation mode (0, 1) for slave-select line of the SPI master.
'''+           0: SPI master activates slave-select line automatically.
'''+           1: slave-select lines are activated manually.
'''ss_time: Only with automatic slave-select activation: 
'''+ Additional time interval of slave-select signal in units (0..255) of 20 ns. Default is 25 (=0.5 s).
'''+ The slave-select signal starts a defined time interval before the clock signal
'''+ and ends by the same time interval after the end of the clock signal.
'''+ The time interval is a half period (of SPI frequency) plus the additional
'''+ time resulting from ss_time.
'''+ The total time interval is dT = 0.5 / f_SPI + ss_time * 20ns.
'''****************************************************************************
Sub SPI_Master_Config(channel, clk_factor, miso_delay, ss_manual, ss_time)
  out(380FH+channel,clk_factor)  
  out(38DFh + channel,miso_delay)
  out(382FH+channel,shift_left(ss_manual,1)) 
  if (ss_manual=0) then
    out(38E7h + channel, ss_time)
  else
    out(38E7h + channel, 0)
  endif
endsub

'******************************************************************************
'''Reads a received SPI message (MISO) of up to 64 bit length from SPI master input register.
'''channel: SPI channel (1..2) 
'''miso_high: Upper 32 bit of received SPI message. Must be a variable of data type Long.
'''miso_low:  Lower 32 bit of received SPI message. Must be a variable of data type Long.
'''******************************************************************************
SUB SPI_Master_Get_Value64(channel,miso_high,miso_low)
  miso_low=in(388FH+channel)
  miso_high=in(389FH+channel)
ENDSUB

'******************************************************************************
'''Reads a received SPI message (MISO) of up to 32 bit length from SPI master input register.
'''+ Do use only if SPI bit length is no more than 32 bit.
'''channel: SPI channel (1..2) 
'''******************************************************************************
Function SPI_Master_Get_Value32(channel) as long
  SPI_Master_Get_Value32=in(388FH+channel)
ENDFunction

'******************************************************************************
'''Prepares an SPI message of up to 64 bit as MOSI output of an SPI master.
'''+The SPI message is not yet sent.
'''channel: SPI channel (1..2) 
'''mosi_high: Upper 32 bit of SPI message to be sent. Must be a variable of data type Long.
'''mosi_low: Lower 32 bit of SPI message to be sent. Must be a variable of data type Long.
'''******************************************************************************
SUB SPI_Master_Set_Value64(channel, mosi_high, mosi_low)
  out(386FH+channel,mosi_low)
  out(387FH+channel,mosi_high)
ENDSUB

'******************************************************************************
'''Prepares an SPI message of up to 32 bit as MOSI output of an SPI master.
'''+The SPI message is not yet sent.
'''+ Do use only if SPI bit length is no more than 32 bit.
'''channel: SPI channel (1..2) 
'''value: 32 bit value to be sent as SPI message. Must be a variable of data type Long.
'''******************************************************************************
SUB SPI_Master_Set_Value32(channel, value)
  out(386FH+channel,value)
ENDSUB

'******************************************************************************
'''Inserts several wait times after a number of cycles into the clock signal of an SPI master.
'''+Time units are based on the clock signal frequency f_SPI.
'''channel: SPI channel (1..2) 
'''clocks: Number (1..16) of cycles after which a wait time is inserted.
'''+        4 means to insert a wait time every 4 cycles.
'''+        0 deactivates the waiting times.
'''half_clk_wait[]: Array holding 5 wait times, the number (1..16) of which is given in half periods.
'''+A wait time t_wait results to: t_wait = 0.5 * half_clk_wait[n] / f_SPI.
'''+If SPI bit length and the clocks parameter result to 6 or more wait times, the value
'''+of half_clk_wait[5] will be repeated.
'''+ Example: clocks=8, half_clk_wait[1]=4
'''+          0.5*4 -> 2 wait periods: _-_-_-_-_-_-_-_-_____-_-_-_-_...
'''******************************************************************************
SUB SPI_Master_Set_Clk_Wait(channel, clocks, half_clk_wait[])
  out(384Fh+channel,clocks-1)
 
  if (channel<3) then  
    out(385Fh+channel,half_clk_wait[1] - 1)
    out(3861h+channel,half_clk_wait[2] - 1)
    out(3863h+channel,half_clk_wait[3] - 1)
    out(3865h+channel,half_clk_wait[4] - 1)
    out(3867h+channel,half_clk_wait[5] - 1)
  else
    out(395Fh+channel-2,half_clk_wait[1] - 1)
    out(3961h+channel-2,half_clk_wait[2] - 1)
    out(3963h+channel-2,half_clk_wait[3] - 1)
    out(3965h+channel-2,half_clk_wait[4] - 1)
    out(3967h+channel-2,half_clk_wait[5] - 1)
  endif  

ENDSUB

'******************************************************************************
'''Returns whether the data transfer of the SPI master is finished. With automatic
'''+ slave-select activation, the slave select line is active during data transfer
'''+ and inactive otherwise.
'''+ ret_val = 0: data transfer is complete (slave select line is inactive).
'''+ ret_val = 1: data transfer is running (slave select line is active).
'''+As long as data transfer is running, no SPI command must be sent to the SPI master.
'''channel: SPI channel (1..2) 
'''******************************************************************************
FUNCTION SPI_Master_Status(channel) AS LONG
  SPI_Master_Status=in(37FFH+channel)
ENDFUNCTION

'******************************************************************************
'''Reads static input channel.
'''+ret_val = 0: data channel has TTL level low.
'''+ret_val = 1: data channel has TTL level high.
'''channel: SPI channel (1..2) 
'''******************************************************************************
Function SPI_Master_Get_Static_Input(channel) as long
  SPI_Master_Get_Static_Input=in(38AFH+channel)
Endfunction

'<ENDREGION>


